/*****************************************************************************/
/* These code snippets are from the Konkoly Observatory 1-m RCC telescope    */
/* control system software. These pure ANSI C functions demonstrate how the  */
/* isotropic/anisotropic pointing model function library (see pointing.c and */
/* pointing.h) can be integrated in a fully operating TCS.		     */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* (c) 2014-2015; Pal, A. <apal@szofi.net>				     */
/*****************************************************************************/

#include <math.h>
#include "pointing.h"

/*****************************************************************************/

static double pointing_coefficients_1[] =
 {	  0.000166,  0.000463,  0.000861,  0.000573,  0.000645, -0.000347,
	  0.000083,  0.000008,  0.000088, -0.000008,  0.000004, -0.000024,
	 -0.000023,  0.000011, -0.000017, -0.000077,  0.000066,       0.0,
               0.0, -0.000117, -0.000035,  0.000191,  0.000037, -0.000445
 };

/* convert raw encoder (tau,dec) values to celestial (tau,dec) */
static int mount_position_corr(double *tau,double *dec)
{
 *tau -= 0.20;
 pointing_anisotropic(tau,dec,pointing_coefficients_1,+1.0,1);
 pointing_refraction_correction_reverse(tau,dec,CRIORCC_LATITUDE,0.90); 
 return(0);
}
/* convert celestial (tau,dec) values to raw encoder (tau,dec) */
static int mount_position_invc(double *tau,double *dec)
{
 pointing_refraction_correction_apparent(tau,dec,CRIORCC_LATITUDE,0.90); 
 pointing_anisotropic(tau,dec,pointing_coefficients_1,-1.0,1);
 *tau += 0.20;
 return(0);
}

/*****************************************************************************/

static int rcc_mount_position(int flags,double *rra,double *rdec,double *rfr)
{
 int	reply,sock;
 struct	criorcc_answer	ca;
 struct	mcc_internal	mcc;
 struct	timeval		tv;
 double	tau,dec,fr,ra;

 mod_criorcc_get_status(&reply,&sock,NULL,&tv,&ca,&mcc);
 if ( reply != 0 || sock<0 )
	return(-1);

 ca_decode_tau_dec(&ca,&tau,&dec);
 mount_position_corr(&tau,&dec);
 
 if ( dec<=90.0 )
	fr=0.0;
 else
  {	fr=180.0;
	dec=180.0-dec;
	tau+=180.0;
  }

 ra=cc_true_greenwich_sidereal_time(cc_julian_day_from_unix(&tv))+CRIORCC_LONGITUDE-tau;
 normalize_angle_unsigned(&ra);

 if ( rra  != NULL )	*rra=ra;
 if ( rdec != NULL )	*rdec=dec;
 if ( rfr  != NULL )	*rfr=fr;

 return(0);
}

/*****************************************************************************/

static int rcc_mount_slew(int flags,double tra,double tdec,double tfr)
{
 int	w,cmd,reply;
 double	tau,dec,stau,sdec;

 /* stop slewing: */
 if ( flags<0 )
  {	w=rcc_mount_stop();
	return(w);
  }

 if ( flags & 0x001 )
  {	double	ra0,dec0;
	if ( rcc_mount_position(0,&ra0,&dec0,NULL)<0 )
		return(-1);
	tra += ra0;
	tdec+= dec0;
  }

 tau=cc_true_greenwich_sidereal_time(cc_current_julian_day())+CRIORCC_LONGITUDE-tra;
 while ( tau<-180.0 )	tau+=360.0;
 while ( 180.0<=tau )	tau-=360.0;
 dec=tdec;

 mount_position_invc(&tau,&dec);
 normalize_angle_signed(&tau);

 cmd=CRIORCC_IPC_CMD_MOUNT_SLEW;
 stau = tau;
 sdec = dec;

 hwrite(rcc->peer,&cmd,sizeof(int));
 hwrite(rcc->peer,&stau,sizeof(double));
 hwrite(rcc->peer,&sdec,sizeof(double));

 hread(rcc->peer,&reply,sizeof(int));

 return(0);
}

/*****************************************************************************/

